@@ -1,11 +1,11 @@
 from __future__ import annotations
 
-from typing import Dict, Optional, Union, Tuple, Sequence, Mapping, Type, Any, TYPE_CHECKING
+import inspect
+from typing import Any, Dict, Mapping, Optional, Sequence, TYPE_CHECKING, Tuple, Type
 
-from jschon.json import JSON, AnyJSONCompatible
+from jschon.json import JSON, JSONCompatible
 from jschon.jsonschema import JSONSchema, Scope
 from jschon.uri import URI
-from jschon.utils import tuplify
 
 if TYPE_CHECKING:
     from jschon.catalog import Catalog
@@ -15,6 +15,7 @@ __all__ = [
     'Vocabulary',
     'Keyword',
     'KeywordClass',
+    'ApplicatorMixin',
     'Applicator',
     'ArrayApplicator',
     'PropertyApplicator',
@@ -33,7 +34,7 @@ class Metaschema(JSONSchema):
     def __init__(
             self,
             catalog: Catalog,
-            value: Mapping[str, AnyJSONCompatible],
+            value: Mapping[str, JSONCompatible],
             core_vocabulary: Vocabulary,
             *default_vocabularies: Vocabulary,
             **kwargs: Any,
@@ -43,7 +44,7 @@ class Metaschema(JSONSchema):
         self.kwclasses: Dict[str, KeywordClass] = {}
         super().__init__(value, catalog=catalog, session='__meta__', **kwargs)
 
-    def _bootstrap(self, value: Mapping[str, AnyJSONCompatible]) -> None:
+    def _bootstrap(self, value: Mapping[str, JSONCompatible]) -> None:
         super()._bootstrap(value)
         self.kwclasses.update(self.core_vocabulary.kwclasses)
         if "$vocabulary" not in value:
@@ -65,15 +66,14 @@ class Vocabulary:
 
 class Keyword:
     key: str = ...
-    types: Optional[Union[str, Tuple[str, ...]]] = None
-    depends: Optional[Union[str, Tuple[str, ...]]] = None
+    types: Tuple[str, ...] = ()
+    depends: Tuple[str, ...] = ()
+    static: bool = False
 
-    def __init__(self, parentschema: JSONSchema, value: AnyJSONCompatible):
-        self.applicator_cls = None
-        for applicator_cls in (Applicator, ArrayApplicator, PropertyApplicator):
-            if isinstance(self, applicator_cls):
-                if (kwjson := applicator_cls.jsonify(parentschema, self.key, value)) is not None:
-                    self.applicator_cls = applicator_cls
+    def __init__(self, parentschema: JSONSchema, value: JSONCompatible):
+        for base_cls in inspect.getmro(self.__class__):
+            if issubclass(base_cls, ApplicatorMixin):
+                if (kwjson := base_cls.jsonify(parentschema, self.key, value)) is not None:
                     break
         else:
             kwjson = JSON(value, parent=parentschema, key=self.key)
@@ -82,10 +82,10 @@ class Keyword:
         self.parentschema: JSONSchema = parentschema
 
     def can_evaluate(self, instance: JSON) -> bool:
-        if self.types is None or instance.type in (types := tuplify(self.types)):
+        if not self.types or instance.type in self.types:
             return True
 
-        if instance.type == "number" and "integer" in types:
+        if instance.type == "number" and "integer" in self.types:
             return instance.data == int(instance.data)
 
         return False
@@ -100,18 +100,19 @@ class Keyword:
 KeywordClass = Type[Keyword]
 
 
-def _is_schema_compatible(value: Any) -> bool:
-    return (isinstance(value, bool) or
-            isinstance(value, Mapping) and all(isinstance(k, str) for k in value))
+class ApplicatorMixin:
+    @classmethod
+    def jsonify(cls, parentschema: JSONSchema, key: str, value: JSONCompatible) -> Optional[JSON]:
+        raise NotImplementedError
 
 
-class Applicator:
+class Applicator(ApplicatorMixin):
     """A :class:`~Keyword` class mixin that sets up a subschema for
     an applicator keyword."""
 
     @classmethod
-    def jsonify(cls, parentschema: JSONSchema, key: str, value: AnyJSONCompatible) -> Optional[JSONSchema]:
-        if _is_schema_compatible(value):
+    def jsonify(cls, parentschema: JSONSchema, key: str, value: JSONCompatible) -> Optional[JSON]:
+        if isinstance(value, (bool, Mapping)):
             return JSONSchema(
                 value,
                 parent=parentschema,
@@ -121,13 +122,13 @@ class Applicator:
             )
 
 
-class ArrayApplicator:
+class ArrayApplicator(ApplicatorMixin):
     """A :class:`~Keyword` class mixin that sets up an array of subschemas
     for an applicator keyword."""
 
     @classmethod
-    def jsonify(cls, parentschema: JSONSchema, key: str, value: AnyJSONCompatible) -> Optional[JSON]:
-        if isinstance(value, Sequence) and all(_is_schema_compatible(v) for v in value):
+    def jsonify(cls, parentschema: JSONSchema, key: str, value: JSONCompatible) -> Optional[JSON]:
+        if isinstance(value, Sequence):
             return JSON(
                 value,
                 parent=parentschema,
@@ -138,16 +139,13 @@ class ArrayApplicator:
             )
 
 
-class PropertyApplicator:
+class PropertyApplicator(ApplicatorMixin):
     """A :class:`~Keyword` class mixin that sets up property-based subschemas
     for an applicator keyword."""
 
     @classmethod
-    def jsonify(cls, parentschema: JSONSchema, key: str, value: AnyJSONCompatible) -> Optional[JSON]:
-        if isinstance(value, Mapping) and all(
-                isinstance(k, str) and _is_schema_compatible(v)
-                for k, v in value.items()
-        ):
+    def jsonify(cls, parentschema: JSONSchema, key: str, value: JSONCompatible) -> Optional[JSON]:
+        if isinstance(value, Mapping):
             return JSON(
                 value,
                 parent=parentschema,
